/*
 * Copyright (C) 2019 - 2025 Devexperts Solutions IE Limited
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 * If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
import { BehaviorSubject } from 'rxjs';
import { CHART_UUID, CanvasElement } from '../../canvas/canvas-bounds-container';
import { ChartBaseElement } from '../../model/chart-base-element';
import { BackgroundDrawer } from '../../drawers/chart-background.drawer';
import { AreaDrawer } from '../../drawers/chart-type-drawers/area.drawer';
import { BarDrawer } from '../../drawers/chart-type-drawers/bar.drawer';
import { BaselineDrawer } from '../../drawers/chart-type-drawers/baseline.drawer';
import { CandleDrawer } from '../../drawers/chart-type-drawers/candle.drawer';
import { HistogramDrawer as MainChartHistogramDrawer } from '../../drawers/chart-type-drawers/histogram.drawer';
import { LineDrawer } from '../../drawers/chart-type-drawers/line.drawer';
import { ScatterPlotDrawer } from '../../drawers/chart-type-drawers/scatter-plot.drawer';
import { CandleSeriesWrapper } from '../../drawers/data-series-drawers/candle-series-wrapper';
import { ColorCandleDrawer } from '../../drawers/data-series-drawers/color-candle.drawer';
import { DifferenceCloudDrawer } from '../../drawers/data-series-drawers/difference-cloud.drawer';
import { HistogramDrawer } from '../../drawers/data-series-drawers/histogram.drawer';
import { LinearDrawer } from '../../drawers/data-series-drawers/linear.drawer';
import { PointsDrawer } from '../../drawers/data-series-drawers/points.drawer';
import { TextDrawer } from '../../drawers/data-series-drawers/text.drawer';
import { TriangleDrawer } from '../../drawers/data-series-drawers/triangle.drawer';
import { DataSeriesDrawer } from '../../drawers/data-series.drawer';
import { HIT_TEST_PREFIX } from '../../drawers/drawing-manager';
import { HTDataSeriesDrawer } from '../../drawers/ht-data-series.drawer';
import { BaselineModel } from '../../model/baseline.model';
import { keys } from '../../utils/object.utils';
import { ONE_FRAME_MS } from '../../utils/numeric-constants.utils';
import { PriceIncrementsUtils } from '../../utils/price-increments.utils';
import { defaultCandleTransformer, hollowCandleTransformer, lineCandleTransformer, trendCandleTransformer, } from './candle-transformer.functions';
import { deleteCandlesIndex } from './candle.functions';
import { TrendHistogramDrawer } from '../../drawers/data-series-drawers/trend-histogram.drawer';
import { debounce } from '../../utils/performance/debounce.utils';
/**
 * Represents a financial instrument to be displayed on a chart
 * @class
 * @property {string} symbol - The symbol of the instrument
 * @property {string} [description] - The description of the instrument
 * @property {Array<number>} [priceIncrements=[0.01]] - An array of possible price step values on the price axis
 * @property {boolean} [bondFraction] - A flag indicating whether the instrument is a bond fraction or not
 */
export class ChartInstrument {
    constructor() {
        this.symbol = 'MOCK';
        /**
         * defines possible price step values on price axis
         */
        this.priceIncrements = [0.01];
    }
}
export class ChartComponent extends ChartBaseElement {
    constructor(chartModel, canvasModel, config, scale, canvasBoundsContainer, drawingManager, hitTestCanvasModel, canvasInputListener, backgroundCanvasModel, chartPanComponent, paneManager, cursorHandler, dynamicObjects, chartResizeHandler) {
        super();
        this.chartModel = chartModel;
        this.canvasModel = canvasModel;
        this.config = config;
        this.scale = scale;
        this.canvasBoundsContainer = canvasBoundsContainer;
        this.drawingManager = drawingManager;
        this.hitTestCanvasModel = hitTestCanvasModel;
        this.canvasInputListener = canvasInputListener;
        this.paneManager = paneManager;
        this.dynamicObjects = dynamicObjects;
        this.chartResizeHandler = chartResizeHandler;
        this._dataSeriesDrawers = {};
        this.backgroundDrawerPredicateSubject = new BehaviorSubject(true);
        this.strToBarType = (str) => { var _a; return (_a = this.barTypeValues.find(t => t === str)) !== null && _a !== void 0 ? _a : 'candle'; };
        this.addChildEntity(this.chartModel);
        this.registerDefaultCandlesTransformers();
        this.baselineModel = new BaselineModel(this.chartModel, chartPanComponent, this.canvasModel, this.canvasInputListener, this.config, this.canvasBoundsContainer, cursorHandler);
        this.addChildEntity(this.baselineModel);
        //#region main chart drawers
        const hTChartDrawer = new HTDataSeriesDrawer(this._dataSeriesDrawers, this.hitTestCanvasModel, paneManager, () => this.hitTestCanvasModel.hitTestDrawersPredicateSubject.getValue());
        this.drawingManager.addDrawerBefore(hTChartDrawer, HIT_TEST_PREFIX + 'DATA_SERIES', 'HIT_TEST_EVENTS');
        //#endregion
        //#region data series drawers
        this.registerDefaultDataSeriesDrawers();
        //#endregion
        this.backgroundDrawer = new BackgroundDrawer(backgroundCanvasModel, this.config, () => this.backgroundDrawerPredicateSubject.getValue());
        drawingManager.addDrawer(this.backgroundDrawer, 'MAIN_BACKGROUND');
        cursorHandler.setCursorForCanvasEl(CanvasElement.PANE_UUID(CHART_UUID), config.components.chart.cursor);
        this.dataSeriesDrawer = new DataSeriesDrawer(paneManager, this.dataSeriesDrawers);
    }
    /**
     * This method overrides the doActivate method of the parent class and calls it.
     * It does not take any parameters and does not return anything.
     * @protected
     * @returns {void}
     */
    doActivate() {
        super.doActivate();
        // TODO hack, main data series is created before doActivate, so we need to add it manually
        this.dynamicObjects.model.addObject({
            id: this.chartModel.mainCandleSeries.id,
            paneId: this.chartModel.mainCandleSeries.extentComponent.paneUUID,
            model: this.chartModel.mainCandleSeries,
            drawer: this.dataSeriesDrawer,
        });
        this.addRxSubscription(this.paneManager.dataSeriesAddedSubject.subscribe(series => {
            this.dynamicObjects.model.addObject({
                id: series.id,
                paneId: series.extentComponent.paneUUID,
                parentId: series.parentId,
                model: series,
                drawer: this.dataSeriesDrawer,
            });
        }));
        this.addRxSubscription(this.paneManager.dataSeriesRemovedSubject.subscribe(series => {
            this.dynamicObjects.model.removeObject(series.id);
        }));
        // redraw background only when chart is resized
        this.addRxSubscription(this.canvasBoundsContainer.observeAnyBoundsChanged().subscribe(debounce(() => {
            // there is multiple calls on this bounds change on lots of sources,
            // and sometimes this calls mixes with this.chartResizeHandler.canvasResized
            // and this leads to change of background in situations when it's not needed,
            // so on multiple calls in one frame we debounced it to avoid that collisions and steam races
            this.backgroundDrawerPredicateSubject.next(false);
        }, ONE_FRAME_MS)));
        this.addRxSubscription(this.chartResizeHandler.canvasResized.subscribe(() => {
            this.backgroundDrawerPredicateSubject.next(true);
        }));
    }
    /**
     * Registers default candle transformers.
     * @private
     * @function
     * @returns {void}
     */
    registerDefaultCandlesTransformers() {
        this.registerCandlesTransformer('candle', defaultCandleTransformer);
        this.registerCandlesTransformer('trend', trendCandleTransformer);
        this.registerCandlesTransformer('hollow', hollowCandleTransformer);
        this.registerCandlesTransformer('line', lineCandleTransformer);
    }
    get barTypeValues() {
        // @ts-ignore
        return keys(this._dataSeriesDrawers);
    }
    get dataSeriesDrawers() {
        return this._dataSeriesDrawers;
    }
    /**
     * You can use this method to determine logic of visual candle transformation for specified chart type.
     * @param chartType
     * @param transformer
     */
    registerCandlesTransformer(chartType, transformer) {
        this.chartModel.registerCandlesTransformer(chartType, transformer);
    }
    /**
     * You can use this method to modify labels for last candle.
     * @param chartType
     * @param handler
     */
    registerLastCandleLabelHandler(chartType, handler) {
        this.chartModel.registerLastCandleLabelHandler(chartType, handler);
    }
    /**
     * You can use this method to determine chart width calculation for specified chart type.
     * @param chartType
     * @param calculator
     */
    registerCandlesWidthCalculator(chartType, calculator) {
        this.chartModel.registerCandlesWidthCalculator(chartType, calculator);
    }
    /**
     * In future this drawers should have same type as main series
     */
    registerDefaultDataSeriesDrawers() {
        const candleDrawer = new CandleDrawer(this.config.components.chart);
        this.registerDataSeriesTypeDrawer('candle', candleDrawer);
        this.registerDataSeriesTypeDrawer('trend', candleDrawer);
        this.registerDataSeriesTypeDrawer('hollow', candleDrawer);
        this.registerDataSeriesTypeDrawer('bar', new BarDrawer(this.config.components.chart));
        this.registerDataSeriesTypeDrawer('line', new LineDrawer(this.config.components.chart));
        this.registerDataSeriesTypeDrawer('scatterPlot', new ScatterPlotDrawer(this.config.colors.scatterPlot));
        this.registerDataSeriesTypeDrawer('area', new AreaDrawer(this.config.components.chart));
        this.registerDataSeriesTypeDrawer('baseline', new BaselineDrawer(this.baselineModel, this.canvasBoundsContainer));
        this.registerDataSeriesTypeDrawer('histogram', new MainChartHistogramDrawer(this.config.components.chart.histogram));
        const chartPaneId = CanvasElement.PANE_UUID(CHART_UUID);
        const mainChartBoundsProvider = () => this.canvasBoundsContainer.getBounds(chartPaneId);
        this.registerDataSeriesTypeDrawer('LINEAR', new LinearDrawer());
        this.registerDataSeriesTypeDrawer('HISTOGRAM', new HistogramDrawer());
        this.registerDataSeriesTypeDrawer('TREND_HISTOGRAM', new TrendHistogramDrawer());
        this.registerDataSeriesTypeDrawer('POINTS', new PointsDrawer());
        this.registerDataSeriesTypeDrawer('COLOR_CANDLE', new CandleSeriesWrapper(new ColorCandleDrawer(this.chartModel), this.config, mainChartBoundsProvider));
        this.registerDataSeriesTypeDrawer('TEXT', new TextDrawer(this.config));
        this.registerDataSeriesTypeDrawer('TRIANGLE', new TriangleDrawer());
        this.registerDataSeriesTypeDrawer('DIFFERENCE', new DifferenceCloudDrawer());
    }
    /**
     * Sets the chart type of main candle series.
     * @param type - new type
     */
    setChartType(type) {
        this.config.components.chart.type = type;
        this.chartModel.rememberCurrentTimeframe();
        this.chartModel.mainCandleSeries.setType(type);
        this.chartModel.mainCandleSeries.updateCandleSeriesColors(Object.assign({}, this.config.colors));
        this.chartModel.mainCandleSeries.recalculateVisualPoints();
        this.chartModel.chartTypeChanged.next(type);
    }
    /**
     * Resets chart scale to default according to config.components.chart.defaultZoomCandleWidth.
     */
    resetChartScale() {
        this.chartModel.doBasicScale();
    }
    /**
     * Sets the timestamp range of the chart by setting the x-axis scale.
     * @param {Timestamp} start - The start timestamp of the range.
     * @param {Timestamp} end - The end timestamp of the range.
     * @param {boolean} forceNoAnimation - true by default
     * @returns {void}
     */
    setTimestampRange(start, end, forceNoAnimation = true) {
        return this.chartModel.setTimestampRange(start, end, forceNoAnimation);
    }
    /**
     * Moves the viewport to exactly xStart..xEnd place.
     * @param xStart - viewport start in units
     * @param xEnd - viewport end in units
     * @param {boolean} forceNoAnimation - true by default
     */
    setXScale(xStart, xEnd, forceNoAnimation = true) {
        return this.scale.setXScale(xStart, xEnd, forceNoAnimation);
    }
    /**
     * Sets the visibility of the wicks in the chart.
     * @param {boolean} isShow - A boolean value indicating whether to show or hide the wicks.
     * @returns {void}
     */
    setShowWicks(isShow) {
        this.config.components.chart.showWicks = isShow;
        this.canvasModel.fireDraw();
    }
    /**
     * Sets the options for the background color applying to the chart axes
     * @param {Object} options - options for both axes
     * @param {boolean} options.x - should the background color apply to the x axis
     * @param {boolean} options.y - should the background color apply to the y axis
     * @returns {void}
     */
    setApplyBackgroundToAxes(options) {
        this.config.components.chart.applyBackgroundToAxes = Object.assign({}, options);
        this.chartModel.bus.fireDraw();
    }
    /**
     * Used to set the main series to chart.
     * @param series
     */
    setMainSeries(series) {
        var _a;
        const instrument = (_a = series.instrument) !== null && _a !== void 0 ? _a : this.chartModel.mainCandleSeries.instrument;
        this.chartModel.mainCandleSeries.instrument = instrument;
        this.chartModel.setAllSeries({
            candles: series.candles,
            instrument,
        }, this.chartModel.getSecondarySeries().map(series => {
            var _a;
            const instrument = (_a = series.instrument) !== null && _a !== void 0 ? _a : this.chartModel.mainCandleSeries.instrument;
            const originalCandles = series.dataPoints;
            // reset the idx of candles to calculate them again
            deleteCandlesIndex(originalCandles);
            return {
                candles: originalCandles,
                instrument,
            };
        }));
        this.updatePriceIncrementsIfNeeded(instrument);
    }
    /**
     * Adds new secondary chart series.
     * @param series
     */
    setSecondarySeries(series) {
        var _a;
        const instrument = (_a = series.instrument) !== null && _a !== void 0 ? _a : this.chartModel.mainCandleSeries.instrument;
        const candleSeriesModel = this.chartModel.setSecondaryCandleSeries(series.candles, instrument);
        this.updatePriceIncrementsIfNeeded(instrument);
        if (candleSeriesModel) {
            return candleSeriesModel;
        }
    }
    /**
     * Sets the main and secondary series in one bulk operation.
     * Reindexing and visual rerender happens at the same time.
     * @param mainSeries
     * @param secondarySeries
     */
    setAllSeries(mainSeries, secondarySeries = []) {
        var _a;
        const instrument = (_a = mainSeries.instrument) !== null && _a !== void 0 ? _a : this.chartModel.mainCandleSeries.instrument;
        this.updatePriceIncrementsIfNeeded(instrument);
        secondarySeries.forEach(series => {
            var _a;
            const instrument = (_a = series.instrument) !== null && _a !== void 0 ? _a : this.chartModel.mainCandleSeries.instrument;
            this.updatePriceIncrementsIfNeeded(instrument);
        });
        this.chartModel.setAllSeries(mainSeries, secondarySeries);
    }
    /**
     * Converts candle index to chart x coordinate
     */
    toXFromCandleIndex(idx) {
        return this.chartModel.toX(idx);
    }
    /**
     * Converts timestamp to chart x coordinate
     */
    toXFromTimestamp(timestamp) {
        const xCandle = this.chartModel.candleFromTimestamp(timestamp);
        return xCandle.xCenter(this.chartModel.scale);
    }
    /**
     * Converts price to chart y coordinate
     */
    toY(price) {
        return this.chartModel.toY(price);
    }
    /**
     * Updates the main and secondary series in one bulk operation.
     * Reindexing and visual rerender happens at the same time.
     * @param mainSeries {CandleSeries}
     * @param secondarySeries {CandleSeries[]}
     */
    updateAllSeries(mainSeries, secondarySeries = []) {
        this.chartModel.updateAllSeries(mainSeries, secondarySeries);
        this.canvasModel.fireDraw();
    }
    /**
     * Removes all data points from the main candle series that are newer than the given timestamp.
     * Can be useful for data replay.
     * @param {Timestamp } timestamp
     */
    removeDataFrom(timestamp) {
        this.chartModel.removeDataFrom(timestamp);
    }
    /**
     * Removes chart candles series.
     * @param {CandleSeriesModel} series
     */
    removeSecondarySeries(series) {
        this.chartModel.removeSecondaryCandleSeries(series);
    }
    /**
     * Adds new candles array to the existing one at the start, mostly used in lazy loading
     * @param target - initial candles array
     * @param prependUpdate - additional candles array, which will be added to the target array
     */
    prependCandles(target, prependUpdate) {
        this.chartModel.prependCandles(target, prependUpdate);
    }
    /**
     * Adds new candle to the chart
     * @param candle - new candle
     * @param instrumentSymbol - name of the instrument to update
     */
    addLastCandle(candle, instrumentSymbol) {
        this.chartModel.addLastCandle(candle, instrumentSymbol);
    }
    /**
     * Remove candle by idx and recalculate indexes
     * @param idx - candle index
     * @param instrumentSymbol - name of the instrument to update
     */
    removeCandleByIdx(idx, instrumentSymbol) {
        this.chartModel.removeCandleByIdx(idx, instrumentSymbol);
    }
    /**
     * Remove candles by ids and recalculate indexes, candles should be as a sequence, follow one by one
     * Works faster than removeCandlesByIds
     *
     * @param ids - candles ids to remove
     * @param selectedCandleSeries - candle series to remove candles from
     */
    removeCandlesByIdsSequence(ids, selectedCandleSeries = this.chartModel.mainCandleSeries) {
        this.chartModel.removeCandlesByIdsSequence(ids, selectedCandleSeries);
    }
    /**
     * Remove candles by ids and recalculate indexes
     *
     * @param ids - candles ids to remove
     * @param selectedCandleSeries - candle series to remove candles from
     */
    removeCandlesByIds(ids, selectedCandleSeries = this.chartModel.mainCandleSeries) {
        this.chartModel.removeCandlesByIds(ids, selectedCandleSeries);
    }
    /**
     * Add candles by id and recalculate indexes
     *
     * @param candles - candles to add
     * @param startId - target candle to start adding candles from
     * @param selectedCandleSeries - candle series to add candles to
     */
    addCandlesById(candles, startId, selectedCandleSeries = this.chartModel.mainCandleSeries) {
        this.chartModel.addCandlesById(candles, startId, selectedCandleSeries);
    }
    /**
     * Updates last candle value
     * @param candle - updated candle
     * @param {string} instrumentSymbol - name of the instrument to update
     */
    updateLastCandle(candle, instrumentSymbol) {
        this.chartModel.updateLastCandle(candle, instrumentSymbol);
    }
    /**
     * Updates candle series for instrument. By default takes main instrument.
     * @param {Array<Candle>} candles
     * @param {string} instrumentSymbol - name of the instrument to update
     */
    updateCandles(candles, instrumentSymbol) {
        this.chartModel.updateCandles(candles, instrumentSymbol);
    }
    /**
     * Sets offsets to viewport.
     * @param offsets - new offsets
     */
    setOffsets(offsets) {
        this.chartModel.setOffsets(offsets);
    }
    /**
     * Returns a SeriesDrawer object based on the provided drawerType.
     * @param {BarType} drawerType - The type of the drawer to be returned.
     * @returns {SeriesDrawer | undefined} - The SeriesDrawer object corresponding to the provided drawerType or undefined if not found.
     */
    getDataSeriesDrawer(drawerType) {
        return this._dataSeriesDrawers[drawerType];
    }
    /**
     * Registers a new chart type drawer or overrides default drawer if drawerType is {BarType}.
     * @param drawerType {string} - a unique name for the drawer, could be {BarType} - in this case will override default drawer for the type
     * @param drawer {SeriesDrawer} - an implementation of the drawer
     */
    registerDataSeriesTypeDrawer(drawerType, drawer) {
        this._dataSeriesDrawers[drawerType] = drawer;
    }
    //#endregion
    /**
     * Updates the price increments of a given instrument if they are not valid or not defined.
     * If the price increments are not valid or not defined, it will set them to a default value.
     * @param {ChartInstrument} instrument - The instrument to update the price increments for.
     */
    updatePriceIncrementsIfNeeded(instrument) {
        if (!instrument.priceIncrements ||
            !PriceIncrementsUtils.validatePriceIncrementsOrPrecisions(instrument.priceIncrements)) {
            instrument.priceIncrements = [
                PriceIncrementsUtils.autoDetectIncrementOfValueRange(this.scale.yEnd - this.scale.yStart),
            ];
        }
    }
    //#region public events
    /**
     * Returns an Observable that emits a void value whenever the offsetsChanged event is triggered.
     * @returns {Observable<void>} An Observable that emits a void value whenever the offsetsChanged event is triggered.
     */
    observeOffsetsChanged() {
        return this.chartModel.offsetsChanged;
    }
    /**
     * Returns an Observable that emits the BarType whenever the chart type is changed.
     * @returns {Observable<BarType>} An Observable that emits the BarType whenever the chart type is changed.
     */
    observeChartTypeChanged() {
        return this.chartModel.chartTypeChanged;
    }
    /**
     * Returns an Observable that emits a void value when the candles in the chart model change.
     * The Observable is obtained by calling the observeCandlesChanged method of the chartModel object.
     * @returns {Observable<void>} An Observable that emits a void value when the candles in the chart model change.
     */
    observeCandlesChanged() {
        return this.chartModel.observeCandlesChanged();
    }
    /**
     * Returns an Observable that emits a void value when the candles are updated in the chart model.
     * The Observable is obtained from the candlesUpdatedSubject of the chartModel.
     * @returns {Observable<void>} Observable that emits a void value when the candles are updated.
     */
    observeCandlesUpdated() {
        return this.chartModel.candlesUpdatedSubject;
    }
    /**
     * Returns an Observable that emits a void value whenever the candlesPrependSubject is triggered.
     * @returns {Observable<PrependedCandlesData>} An Observable that emits a PrependedCandlesData whenever the candles are added to the end.
     */
    observeCandlesPrepended() {
        return this.chartModel.candlesPrependSubject;
    }
}
