/*
 * Copyright (C) 2019 - 2025 Devexperts Solutions IE Limited
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 * If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
import { cloneUnsafe } from '../../utils/object.utils';
export class SecondaryChartColorsPool {
    constructor(config) {
        this.config = config;
        this.usedChartColors = {};
        this.symbolsToColorKeys = {};
        this.secondaryChartColorsPool = this.createColorPool(this.config.colors.secondaryChartTheme);
        this.usedChartColors = Object.keys(this.secondaryChartColorsPool).reduce((acc, key) => (Object.assign(Object.assign({}, acc), { [key]: 0 })), {});
    }
    /**
     * Creates a color pool object based on the provided color array
     * @param {SecondaryChartTheme[]} colorArray - An array of SecondaryChartTheme objects
     * @returns {Object} - An object containing a key-value pair for each color in the colorArray
     * The key is a string in the format "colorX" where X is the index of the color in the array
     * The value is an object containing the FullChartColors for the color, with lineTheme and areaTheme properties
     */
    createColorPool(colorArray) {
        return colorArray.reduce((acc, item, index) => {
            acc[`color${index}`] = Object.assign(Object.assign({}, cloneUnsafe(this.config.colors)), { lineTheme: Object.assign({}, item.lineTheme), areaTheme: Object.assign({}, item.areaTheme) });
            return acc;
        }, {});
    }
    /**
     * Takes a color from the secondaryChartColorsPool and assigns it to a symbol.
     * @param {string} symbol - The symbol to assign the color to.
     * @returns {string} The color assigned to the symbol.
     */
    takeColorFromPool(symbol) {
        let minColors = Number.MAX_SAFE_INTEGER;
        let minColorKey = '';
        for (const [key, value] of Object.entries(this.usedChartColors)) {
            if (value < minColors) {
                minColors = value;
                minColorKey = key;
            }
        }
        this.usedChartColors[minColorKey] = this.usedChartColors[minColorKey] + 1;
        this.symbolsToColorKeys[symbol] = minColorKey;
        return this.secondaryChartColorsPool[minColorKey];
    }
    /**
     * Adds a color to the chart color pool.
     *
     * @param {string} symbol - The symbol of the color to be added.
     *
     * @returns {void}
     */
    addColorToPool(symbol) {
        const key = this.symbolsToColorKeys[symbol];
        if (this.usedChartColors[key]) {
            this.usedChartColors[key] = this.usedChartColors[key] - 1;
        }
    }
    /**
     * Updates the color configuration for a given symbol in the chart.
     * @param {string} symbol - The symbol for which the color configuration needs to be updated.
     * @param {CandleSeriesColors} config - The new color configuration for the symbol.
     * @returns {void}
     */
    updateColorConfig(symbol, config) {
        var _a, _b;
        const existColorConfig = this.secondaryChartColorsPool[this.usedChartColors[symbol]];
        if (existColorConfig) {
            const colors = config;
            this.secondaryChartColorsPool[this.usedChartColors[symbol]] = Object.assign(Object.assign({}, config), { lineTheme: Object.assign(existColorConfig.lineTheme, (_a = colors.lineTheme) !== null && _a !== void 0 ? _a : {}), areaTheme: Object.assign(existColorConfig.areaTheme, (_b = colors.areaTheme) !== null && _b !== void 0 ? _b : {}) });
        }
    }
}
