/*
 * Copyright (C) 2019 - 2025 Devexperts Solutions IE Limited
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 * If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */
import { CanvasElement, X_AXIS_MOBILE_PADDING } from '../../../canvas/canvas-bounds-container';
import { avoidAntialiasing, drawRoundedRect } from '../../../utils/canvas/canvas-drawing-functions.utils';
import { priceLabelDrawersMap } from '../../y_axis/price_labels/price-label.drawer';
export class CrossAndLabelsDrawerType {
    constructor(config, canvasBoundsContainer, paneManager, crossDrawPredicate = () => true, noLines) {
        this.config = config;
        this.canvasBoundsContainer = canvasBoundsContainer;
        this.paneManager = paneManager;
        this.crossDrawPredicate = crossDrawPredicate;
        this.noLines = noLines;
    }
    /**
     * Draws a cross tool on a canvas context and its labels if the crossDrawPredicate is true.
     * @param {CanvasRenderingContext2D} ctx - The canvas context to draw on.
     * @param {CrossToolHover} hover - The hover object containing information about the cross tool's position.
     */
    draw(ctx, hover) {
        if (this.crossDrawPredicate()) {
            avoidAntialiasing(ctx, () => this.drawCrossTool(ctx, hover));
        }
    }
    /**
     * This is a protected method called `drawCrossTool` that takes two parameters: `ctx` of type `CanvasRenderingContext2D` and `hover` of type `CrossToolHover`.
     * The method draws a cross tool on the canvas using the provided context. It first gets the bounds of all panes and the hit test bounds of all panes. It then gets the top padding of the x-axis from the configuration and the left padding of the y-axis from the configuration based on the type of y-label.
     * If the hit test bounds of all panes contain the hover coordinates, it draws a horizontal line and a vertical line using the provided coordinates and the bounds of all panes. It sets the stroke style to the line color from the configuration and sets the line dash to the line dash from the configuration. It then begins a new path, moves to the start of the horizontal line, draws a line to the end of the horizontal line, moves to the start of the vertical line, and draws a line to the end of the vertical line. Finally, it strokes the path.
     */
    drawCrossTool(ctx, hover) {
        var _a, _b, _c, _d, _e;
        const allPanes = this.canvasBoundsContainer.getBounds(CanvasElement.ALL_PANES);
        // extension is need for allPanes.y to fit into hit test
        const allPanesHT = this.canvasBoundsContainer.getBoundsHitTest(CanvasElement.ALL_PANES, { extensionY: 0.0001 });
        const xAxisTopPadding = (_a = this.config.components.xAxis.padding.top) !== null && _a !== void 0 ? _a : 0;
        const padding = (_c = this.config.components.yAxis.typeConfig[(_b = this.config.components.crossTool.yLabel.type) !== null && _b !== void 0 ? _b : 'badge']) === null || _c === void 0 ? void 0 : _c.paddings;
        const yAxisLeftPadding = (_d = padding === null || padding === void 0 ? void 0 : padding.start) !== null && _d !== void 0 ? _d : 0;
        // we want to draw hover even if y coordinate is beyond chart
        // Example: snap option is turned on, and chart is scrolled down, so it's not visible
        const paneHT = (_e = this.paneManager.panes[hover.paneId]) === null || _e === void 0 ? void 0 : _e.ht;
        if (allPanesHT(hover.x, allPanes.y)) {
            const horizontalLineCoords = {
                start: [allPanes.x, hover.y],
                end: [allPanes.x + allPanes.width + yAxisLeftPadding, hover.y],
            };
            const verticalLineCoords = {
                start: [hover.x, allPanes.y],
                end: [hover.x, allPanes.y + allPanes.height + xAxisTopPadding],
            };
            const allowHorizontal = paneHT === null || paneHT === void 0 ? void 0 : paneHT(hover.x, hover.y);
            if (!this.noLines) {
                ctx.strokeStyle = this.config.colors.crossTool.lineColor;
                ctx.setLineDash(this.config.components.crossTool.lineDash);
                ctx.beginPath();
                // don't draw horizontal line if it's not in hit test
                if (allowHorizontal) {
                    ctx.moveTo(...horizontalLineCoords.start);
                    ctx.lineTo(...horizontalLineCoords.end);
                }
                ctx.moveTo(...verticalLineCoords.start);
                ctx.lineTo(...verticalLineCoords.end);
                ctx.stroke();
            }
            allowHorizontal && this.drawYLabel(ctx, hover);
            this.drawXLabel(ctx, hover);
        }
    }
    /**
     * Draws the X axis label for the cross tool on the canvas.
     * @param {CanvasRenderingContext2D} ctx - The 2D rendering context for the canvas.
     * @param {CrossToolHover} hover - The hover object containing the x and y coordinates of the cross tool.
     * @returns {void}
     */
    drawXLabel(ctx, hover) {
        var _a, _b, _c, _d, _e;
        const xLabelPadding = this.config.components.crossTool.xLabel.padding;
        const xLabelMargin = this.config.components.crossTool.xLabel.margin;
        const crossToolColors = this.config.colors.crossTool;
        // X axis label same for all
        if (this.config.components.xAxis.visible) {
            ctx.font = this.config.components.xAxis.fontSize + 'px ' + this.config.components.xAxis.fontFamily;
            // X hover
            const x = hover.x;
            const labelText = hover.time;
            const width = ctx.measureText(labelText).width;
            const fontHeight = this.config.components.xAxis.fontSize;
            // box
            const xAxis = this.canvasBoundsContainer.getBounds(CanvasElement.X_AXIS);
            ctx.save();
            ctx.fillStyle = crossToolColors.labelBoxColor;
            const xLabelPaddingLeft = (_a = xLabelPadding === null || xLabelPadding === void 0 ? void 0 : xLabelPadding.left) !== null && _a !== void 0 ? _a : 0;
            const xLabelPaddingRight = (_b = xLabelPadding === null || xLabelPadding === void 0 ? void 0 : xLabelPadding.right) !== null && _b !== void 0 ? _b : 0;
            const xLabelPaddingTop = (_c = xLabelPadding === null || xLabelPadding === void 0 ? void 0 : xLabelPadding.top) !== null && _c !== void 0 ? _c : 0;
            const xLabelPaddingBottom = (_d = xLabelPadding === null || xLabelPadding === void 0 ? void 0 : xLabelPadding.bottom) !== null && _d !== void 0 ? _d : 0;
            const xLabelMarginTop = (_e = xLabelMargin === null || xLabelMargin === void 0 ? void 0 : xLabelMargin.top) !== null && _e !== void 0 ? _e : 0;
            const boxWidth = width + xLabelPaddingLeft + xLabelPaddingRight;
            const boxHeight = fontHeight + xLabelPaddingTop + xLabelPaddingBottom;
            const xBoxPos = Math.max(x - boxWidth / 2, 0);
            const yBoxPos = xAxis.y + xLabelMarginTop + X_AXIS_MOBILE_PADDING / 2;
            drawRoundedRect(ctx, xBoxPos, yBoxPos, boxWidth, boxHeight);
            // label
            ctx.fillStyle = crossToolColors.labelTextColor;
            const xTextPos = Math.max(x - width / 2, xLabelPaddingLeft);
            const yTextPos = xAxis.y + xLabelMarginTop + fontHeight + X_AXIS_MOBILE_PADDING / 2 + xLabelPaddingTop - 1; // -1 for vertical adjustment
            ctx.fillText(labelText, xTextPos, yTextPos);
            ctx.restore();
        }
    }
    /**
     * Draws the Y-axis label on the canvas context at the given point.
     * @param {CanvasRenderingContext2D} ctx - The canvas context to draw on.
     * @param {Point} point - The point where the label should be drawn.
     * @returns {void}
     */
    drawYLabel(ctx, point) {
        const yLabelPadding = this.config.components.crossTool.yLabel.padding;
        const crossToolColors = this.config.colors.crossTool;
        // Y axis label different for main chart pane and the rest panes
        if (this.config.components.yAxis.visible) {
            const pane = this.paneManager.panes[point.paneId];
            const y = point.y;
            const type = this.config.components.crossTool.yLabel.type;
            if (!pane) {
                return;
            }
            for (const extent of pane.yExtentComponents) {
                const price = extent.regularValueFromY(y);
                const label = extent.valueFormatter(price);
                const bounds = this.canvasBoundsContainer.getBounds(CanvasElement.PANE_UUID_Y_AXIS(pane.uuid, extent.idx));
                const drawYLabel = priceLabelDrawersMap[type];
                const textColor = type === 'plain' ? crossToolColors.lineColor : crossToolColors.labelTextColor;
                drawYLabel(ctx, bounds, label, y, {
                    textColor,
                    bgColor: crossToolColors.labelBoxColor,
                    paddingBottom: yLabelPadding === null || yLabelPadding === void 0 ? void 0 : yLabelPadding.bottom,
                    paddingEnd: yLabelPadding === null || yLabelPadding === void 0 ? void 0 : yLabelPadding.end,
                    paddingTop: yLabelPadding === null || yLabelPadding === void 0 ? void 0 : yLabelPadding.top,
                }, extent.yAxis.state, this.config.colors.yAxis, true);
            }
        }
    }
}
